﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/02/10 初版
//
// FHC3313 Sample Program
// -----------------------------------------------------------------
namespace FHC3313_Sample
{
    public partial class Form_FHC3313_SAMPLE : Form
    {

        //画像メモリ格納領域
        private FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();
        //FHC3313クラス
        private FVIL.Video.CFviVideoFHC3313 m_video = new FVIL.Video.CFviVideoFHC3313();
        //連続取り込み用画像メモリ格納領域
        private List<FVIL.Data.CFviImage> m_ImageList = new List<FVIL.Data.CFviImage>();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();
        //WAIT_FRAME構造体
        private FVIL.Video.CFviWaitFrame m_WaitFrame = new FVIL.Video.CFviWaitFrame();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;

        //画像オブジェクト番号
        public int m_intBufImgIndex;
        //フレーム番号
        public uint m_uintFrameNo;

        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;
        //フリーランの状態
        private bool m_flgFreerun;
        
        public Form_FHC3313_SAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            dialogLicence licence_dialog = new dialogLicence();
            if ((licence_dialog.ShowDialog(this) != DialogResult.OK))
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;
            m_flgFreerun = false;
            m_intBufImgIndex = 0;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0.000";
            //フレーム数テキストボックス初期化
            TextBoxFreame.Text = "0";
            //キャプチャー数テキストボックス初期化
            TextBoxSetCaptureNum.Text = "10";
            //画像メモリ確保数テキストボックス初期化
            TextBoxImageLockNum.Text = "5";
            //画像オブジェクト番号テキストボックス初期化
            TextBoxBufImgIndex.Text = m_intBufImgIndex.ToString();

            //エンコーダパラメータコンボボックス初期化
            ComboBoxMode.Items.Clear();
            ComboBoxMode.Items.Add("エンコーダスキャンモード");
            ComboBoxMode.Items.Add("エンコーダライン選択モード");
            ComboBoxMode.SelectedIndex = 0;

            ComboBoxStart.Items.Clear();
            ComboBoxStart.Items.Add("CPU");
            ComboBoxStart.Items.Add("外部トリガ");
            ComboBoxStart.SelectedIndex = 0;

            ComboBoxPhase.Items.Clear();
            ComboBoxPhase.Items.Add("AB相");
            ComboBoxPhase.Items.Add("A相");
            ComboBoxPhase.SelectedIndex = 0;

            ComboBoxDirection.Items.Clear();
            ComboBoxDirection.Items.Add("CW");
            ComboBoxDirection.Items.Add("CCW");
            ComboBoxDirection.SelectedIndex = 0;
            
            ComboBoxZEnable.Items.Clear();
            ComboBoxZEnable.Items.Add("使用しない");
            ComboBoxZEnable.Items.Add("使用する");
            ComboBoxZEnable.SelectedIndex = 0;

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.SelectedIndex = 0;

            //エンコーダーモードチェック時のみパラメータ変更化
            GroupEncorderParam.Enabled = CheckEncoderMode.Checked;
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイル設定画面
            Form_BROWSEIN formbroseini = new Form_BROWSEIN();

            //iniファイルの選択
            m_strIniFilePath = "";
            if (formbroseini.ShowDialog(this) == DialogResult.OK)
            {
                m_strIniFilePath = formbroseini.strIniFilePath;
            }

            if (m_strIniFilePath.Length > 0)
            {
                try
                {
                    //ビデオデバイスがオープンされていた場合はクローズする
                    if (m_video.IsOpen())
                    {
                        m_video.Close();
                        m_flgVideoOpened = false;
                    }

                    //デバイスオープン
                    m_video.Open(-1, -1);
                    m_flgVideoOpened = true;

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);

                    //iniファイルをコンポーネントに設定
                    if (m_strIniFilePath.Length > 0)
                    {
                        m_video.LoadIniFile(m_strIniFilePath);
                    }

                    //取込画像を正規化
                    m_video.ChangeImageSize(m_image);

                    //初期値を表示
                    //ラインセンサモード取得
                    uint LineSensorMode = 0, EncoderMode = 0;
                    m_video.GetLineSensorMode(ref LineSensorMode, ref EncoderMode);
                    if (LineSensorMode == 1)
                    {
                        CheckLineSensorMode.Checked = true;
                    }
                    else
                    {
                        CheckLineSensorMode.Checked = false;
                    }
                    if (EncoderMode == 1)
                    {
                        CheckEncoderMode.Checked = true;
                    }
                    else
                    {
                        CheckEncoderMode.Checked = false;
                    }
                    
                    //エンコーダパラメータ取得
                    uint Mode = 0, Start = 0, Phase = 0, Direction = 0, ZEnable = 0, Comp1 = 0, Comp2 = 0;
                    m_video.GetEncoderParam(ref Mode, ref Start, ref Phase, ref Direction, ref ZEnable, ref Comp1, ref Comp2);
                    ComboBoxMode.SelectedIndex = (int)Mode;
                    ComboBoxStart.SelectedIndex = (int)Start;
                    ComboBoxPhase.SelectedIndex = (int)Phase;
                    ComboBoxDirection.SelectedIndex = (int)Direction;
                    ComboBoxZEnable.SelectedIndex = (int)ZEnable;
                    NumericUpDownComp1.Value = Comp1;
                    NumericUpDownComp2.Value = Comp2;

                    //トリガリトライ期間取得
                    uint CycleTime = 0;
                    ComboBoxTrigger.SelectedIndex = (int)m_video.Trigger;
                    NumericUpDownExposure.Value = (int)m_video.ExposureTime;
                    m_video.GetTriggerCycleTime(ref CycleTime);
                    NumericUpDownCycleTime.Value = (int)CycleTime;

                    //エンコーダモード時、エンコーダパラメータ変更可
                    GroupEncorderParam.Enabled = CheckEncoderMode.Checked;

                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;

            try
            {
                if (fnSetExpCycleTime())
                {
                    //画像メモリロック
                    m_video.ContinuousGrab.MemoryLock(m_image);
                    // 描画imageのセット
                    cFviImageView1.Image = m_image;
                    //測定時間表示有効
                    TextBoxTime.Enabled = true;
                    //時間測定開始
                    m_timer.Start();
                    //カメラ単発取り込み
                    m_video.ContinuousGrab.GrabSync(1);
                    //時間測定終了
                    msec = m_timer.Stop();
                    //測定時間表示
                    TextBoxTime.Text = msec.ToString("0.000");
                    //再描画
                    cFviImageView1.Refresh();
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {
            //ボタン表示変更
            if (m_flgFreerun == true)
            {
                //画像取り込み中止
                m_video.ContinuousGrab.StopCapture();
                //ステータス変更
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //トリガパラメータ変更可
                GroupTrigger.Enabled = true;
            }
            else
            {
                try
                {
                    if (fnSetExpCycleTime())
                    {
                        //ステータス変更
                        ButtonGrabContinue.Text = "Stop";
                        ButtonGrabOne.Enabled = false;
                        m_flgFreerun = true;
                        //トリガパラメータ変更不可
                        GroupTrigger.Enabled = false;
                        //測定時間表示無効
                        TextBoxTime.Text = "0.000";
                        TextBoxTime.Enabled = false;

                        //画像メモリ格納領域クリア
                        m_ImageList.Clear();
                        uint ImageLockNum = uint.Parse(TextBoxImageLockNum.Text);
                        for (int LockNum = 0; LockNum < ImageLockNum; LockNum++)
                        {
                            //画像メモリ確保数分メモリ格納領域追加
                            m_ImageList.Add(new FVIL.Data.CFviImage());
                            //取込画像を正規化
                            m_video.ChangeImageSize(m_ImageList[LockNum]);
                        }
                        //画像メモリ格納領域をセット
                        m_video.ContinuousGrab.Images = m_ImageList;
                        //画像メモリロック
                        m_video.ContinuousGrab.MemoryLock(ImageLockNum);

                        timer1.Interval = 50;
                        timer1.Start();
                        //連続カメラ取り込み
                        uint CaptureNum = uint.Parse(TextBoxSetCaptureNum.Text);
                        m_video.ContinuousGrab.GrabASync(CaptureNum);
                    }
                    else
                    {
                        //カメラ露光時間、サイクルタイムの設定失敗の為、何もせず終了する。
                        return;
                    }
                }
                catch (FVIL.CFviException ex)
                {
                    //ステータス変更
                    ButtonGrabContinue.Text = "Grab Continue";
                    ButtonGrabOne.Enabled = true;
                    m_flgFreerun = false;
                    timer1.Enabled = false;
                    //トリガパラメータ変更可
                    GroupTrigger.Enabled = true;

                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //ステータス変更
                    ButtonGrabContinue.Text = "Grab Continue";
                    ButtonGrabOne.Enabled = true;
                    m_flgFreerun = false;
                    timer1.Enabled = false;
                    //トリガパラメータ変更可
                    GroupTrigger.Enabled = true;

                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            Application.DoEvents();

        }

        /// <summary>
        /// タイマーイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// タイマーによる表示更新。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            try
            {
                if (m_flgFreerun == true)
                {
                    uint CaptureNum = uint.Parse(TextBoxSetCaptureNum.Text);
                    m_WaitFrame = m_video.ContinuousGrab.QueryCaptureNo();
                    if ((CaptureNum > m_WaitFrame.TotalFrameCount) || (CaptureNum == 0))
                    {
                        timer1.Stop();
                        //フレーム数表示更新
                        TextBoxFreame.Text = m_WaitFrame.TotalFrameCount.ToString();
                        //最新フレームをセット
                        cFviImageView1.Image = m_ImageList[(int)m_WaitFrame.LastFrameNo];
                        //再描画
                        cFviImageView1.Refresh();
                        int stat = m_video.ContinuousGrab.GrabStatus;
                        timer1.Start();
                    }
                    else
                    {
                        timer1.Stop();
                        //フレーム数表示更新
                        TextBoxFreame.Text = m_WaitFrame.TotalFrameCount.ToString();
                        //最新フレームをセット
                        cFviImageView1.Image = m_ImageList[(int)m_WaitFrame.LastFrameNo];
                        //再描画
                        cFviImageView1.Refresh();
                        //画像取り込み中止
                        m_video.ContinuousGrab.StopCapture();
                        //ステータス変更
                        ButtonGrabContinue.Text = "Grab Continue";
                        ButtonGrabOne.Enabled = true;
                        m_flgFreerun = false;
                        timer1.Enabled = false;
                        //トリガパラメータ変更可
                        GroupTrigger.Enabled = true;
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //画像取り込み中止
                m_video.ContinuousGrab.StopCapture();
                //ステータス変更
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //トリガパラメータ変更可
                GroupTrigger.Enabled = true;

                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //画像取り込み中止
                m_video.ContinuousGrab.StopCapture();
                //ステータス変更
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                m_flgFreerun = false;
                timer1.Enabled = false;
                //トリガパラメータ変更可
                GroupTrigger.Enabled = true;

                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダモードチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダモードに変更します。
        /// </remarks>
        private void CheckEncoderMode_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    uint LineSensorMode, EncoderMode;
                    if (CheckLineSensorMode.Checked)
                    {
                        LineSensorMode = 1;
                    }
                    else
                    {
                        LineSensorMode = 0;
                    }
                    if (CheckEncoderMode.Checked)
                    {
                        EncoderMode = 1;
                    }
                    else
                    {
                        EncoderMode = 0;
                    }
                    //ラインセンサモード設定
                    m_video.SetLineSensorMode(LineSensorMode, EncoderMode);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //エンコーダーモードのときのみパラメータ変更化
            GroupEncorderParam.Enabled = CheckEncoderMode.Checked;

        }

        /// <summary>
        /// ラインセンサモードチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ラインセンサモードの設定（エリア　Or ラインセンサを指定する）。
        /// </remarks>
        private void CheckLineSensorMode_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    uint LineSensorMode, EncoderMode;
                    if (CheckLineSensorMode.Checked)
                    {
                        LineSensorMode = 1;
                    }
                    else
                    {
                        LineSensorMode = 0;
                    }
                    if (CheckEncoderMode.Checked)
                    {
                        EncoderMode = 1;
                    }
                    else
                    {
                        EncoderMode = 0;
                    }
                    //ラインセンサモード設定
                    m_video.SetLineSensorMode(LineSensorMode, EncoderMode);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダモードチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダモードの設定( 0:エンコーダスキャンモード 1:エンコーダライン選択モード )。
        /// </remarks>
        private void ComboBoxMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //エンコーダパラメータ設定
                    m_video.SetEncoderParam((uint)ComboBoxMode.SelectedIndex, (uint)ComboBoxStart.SelectedIndex, (uint)ComboBoxPhase.SelectedIndex, (uint)ComboBoxDirection.SelectedIndex, (uint)ComboBoxZEnable.SelectedIndex, (uint)NumericUpDownComp1.Value, (uint)NumericUpDownComp2.Value);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダ起動モードの設定( 0:CPU 1:エンコーダ外部トリガ )。
        /// </remarks>
        private void ComboBoxStart_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //エンコーダパラメータ設定
                    m_video.SetEncoderParam((uint)ComboBoxMode.SelectedIndex, (uint)ComboBoxStart.SelectedIndex, (uint)ComboBoxPhase.SelectedIndex, (uint)ComboBoxDirection.SelectedIndex, (uint)ComboBoxZEnable.SelectedIndex, (uint)NumericUpDownComp1.Value, (uint)NumericUpDownComp2.Value);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// AB相の設定( 0:AB相、1:A相 )。
        /// </remarks>
        private void ComboBoxPhase_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //エンコーダパラメータ設定
                    m_video.SetEncoderParam((uint)ComboBoxMode.SelectedIndex, (uint)ComboBoxStart.SelectedIndex, (uint)ComboBoxPhase.SelectedIndex, (uint)ComboBoxDirection.SelectedIndex, (uint)ComboBoxZEnable.SelectedIndex, (uint)NumericUpDownComp1.Value, (uint)NumericUpDownComp2.Value);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パルスの方向の設定 ( 0:CW、1:CCW )。
        /// </remarks>
        private void ComboBoxDirection_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //エンコーダパラメータ設定
                    m_video.SetEncoderParam((uint)ComboBoxMode.SelectedIndex, (uint)ComboBoxStart.SelectedIndex, (uint)ComboBoxPhase.SelectedIndex, (uint)ComboBoxDirection.SelectedIndex, (uint)ComboBoxZEnable.SelectedIndex, (uint)NumericUpDownComp1.Value, (uint)NumericUpDownComp2.Value);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// Z相使用の設定( 0:使用しない 1:使用する )。
        /// </remarks>
        private void ComboBoxZEnable_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //エンコーダパラメータ設定
                    m_video.SetEncoderParam((uint)ComboBoxMode.SelectedIndex, (uint)ComboBoxStart.SelectedIndex, (uint)ComboBoxPhase.SelectedIndex, (uint)ComboBoxDirection.SelectedIndex, (uint)ComboBoxZEnable.SelectedIndex, (uint)NumericUpDownComp1.Value, (uint)NumericUpDownComp2.Value);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 比較レジスタ1の設定。
        /// </remarks>
        private void NumericUpDownComp1_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //エンコーダパラメータ設定
                    m_video.SetEncoderParam((uint)ComboBoxMode.SelectedIndex, (uint)ComboBoxStart.SelectedIndex, (uint)ComboBoxPhase.SelectedIndex, (uint)ComboBoxDirection.SelectedIndex, (uint)ComboBoxZEnable.SelectedIndex, (uint)NumericUpDownComp1.Value, (uint)NumericUpDownComp2.Value);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// エンコーダパラメータ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 比較レジスタ2の設定。
        /// </remarks>
        private void NumericUpDownComp2_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //エンコーダパラメータ設定
                    m_video.SetEncoderParam((uint)ComboBoxMode.SelectedIndex, (uint)ComboBoxStart.SelectedIndex, (uint)ComboBoxPhase.SelectedIndex, (uint)ComboBoxDirection.SelectedIndex, (uint)ComboBoxZEnable.SelectedIndex, (uint)NumericUpDownComp1.Value, (uint)NumericUpDownComp2.Value);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// ←ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像メモリの番号を戻す。
        /// </remarks>
        private void ButtonLeft_Click(object sender, EventArgs e)
        {
            m_intBufImgIndex--;
            if (m_intBufImgIndex < 0)
            {
                m_intBufImgIndex = int.Parse(TextBoxImageLockNum.Text) - 1;
            }
            if (m_intBufImgIndex < 0)
            {
                m_intBufImgIndex = 0;
            }
            TextBoxBufImgIndex.Text = m_intBufImgIndex.ToString();
            if (m_ImageList.Count > 0)
            {
                //表示画像をセット
                cFviImageView1.Image = m_ImageList[m_intBufImgIndex];
                //再描画
                cFviImageView1.Refresh();
            }
        }

        /// <summary>
        /// →ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像メモリの番号を進める。
        /// </remarks>
        private void ButtonRight_Click(object sender, EventArgs e)
        {
            m_intBufImgIndex++;
            if (m_intBufImgIndex >= int.Parse(TextBoxImageLockNum.Text))
            {
                m_intBufImgIndex = 0;
            }
            TextBoxBufImgIndex.Text = m_intBufImgIndex.ToString();
            if (m_ImageList.Count > 0)
            {
                //表示画像をセット
                cFviImageView1.Image = m_ImageList[m_intBufImgIndex];
                //再描画
                cFviImageView1.Refresh();
            }
        }

        /// <summary>
        /// キャプチャー数テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// キャプチャー数のクランプ処理。
        /// </remarks>
        private void TextBoxSetCaptureNum_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (int.Parse(TextBoxSetCaptureNum.Text) < 0)
                {
                    TextBoxSetCaptureNum.Text = "0";
                }
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は0に初期化する。
                TextBoxSetCaptureNum.Text = "0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 画像メモリ確保数テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像メモリ確保数のクランプ処理。
        /// </remarks>
        private void TextBoxImageLockNum_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (int.Parse(TextBoxSetCaptureNum.Text) < 1)
                {
                    TextBoxSetCaptureNum.Text = "1";
                }
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は1に初期化する。
                TextBoxSetCaptureNum.Text = "1";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。
            cFviImageView1.Display.Magnification = 1.0; //表示倍率の設定
            cFviImageView1.Refresh();                   //再描画

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            cFviImageView1.FitImageSize();                   //表示倍率を調整
            cFviImageView1.Refresh();                        //再描画

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Enable = true;
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Enable = false;
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            //フリーラン中ならば、停止する。
	        if (m_flgFreerun == true) {
		        ButtonGrabContinue.Text = "Grab Continue";
		        ButtonGrabOne.Enabled = true;
		        m_flgFreerun = false;
		        timer1.Enabled = false;
		        Application.DoEvents();
	        }

        	//iniファイル選択
	        fnSetIniFile();

        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch (ex.ErrorCode)
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// カメラ露光時間、サイクルタイムの設定関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// Cycle Time は Exposure Time より大きい必要があります。
        /// </remarks>
        private bool fnSetExpCycleTime()
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    if (NumericUpDownExposure.Value < NumericUpDownCycleTime.Value)
                    {
                        //トリガリトライ期間設定
                        uint CycleTime = (uint)NumericUpDownCycleTime.Value;
                        m_video.SetTriggerCycleTime(CycleTime);
                        m_video.ExposureTime = (uint)NumericUpDownExposure.Value;
                    }
                    else
                    {
                        //エラー処理
                        MessageBox.Show("Cycle Time を Video Exposure より大きくして下さい。", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return(false);
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }
            return(true);
        }


        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC061CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            //ロックされたメモリを解除します
            if (m_video.ContinuousGrab.IsMemoryLocked())
            {
                //取り込みの停止
                m_video.ContinuousGrab.StopCapture();
                m_video.ContinuousGrab.MemoryUnlock();
            }
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideoOpened) 
            {
                m_video.Close();
                m_flgVideoOpened = false;
            }
        }
    }

}